#import "OWOReplacementComponent.h"

// Declare some private methods inside WebObjects to avoid warnings.

@interface WOContext (WebObjectsPrivateMethods)
- (void)_pushComponent:aComponent withAssociations:(NSArray *)associations forKeys:(NSArray *)keys;
- (void)_popComponentAssociationsAndKeys;
@end

@interface WOComponent (WebObjectsPrivateMethods)
- (void)_popFromContextWithAssociations:(NSArray *)associations keys:(NSArray *)keys;
- (void)_pushInContextWithAssociations:(NSArray *)associations keys:(NSArray *)keys;
- (WOComponent *)_subcomponentForElementWithID:foo;
- (void)_setSubcomponent:realElement forElementWithID:elementID;
- (void)_setIsPage:(BOOL)yn;
- (BOOL)_isPage;
@end

@implementation OWOReplacementComponent

- (void)_pushComponentInContext:realElement {

    WOContext *context;
    WOComponent *parent;
    WOComponent *child;

    context = [WOApp context];
    parent = [context component];
    child = [parent _subcomponentForElementWithID:[context elementID]];

    if (child != realElement) {
        if ([realElement respondsToSelector:@selector(_isPage)] && [realElement _isPage])
            [realElement _setIsPage:NO];
        [parent _setSubcomponent:realElement forElementWithID:[context elementID]];
    }

    [realElement _pushInContextWithAssociations:associations keys:keys];
}

- (void)_popComponentFromContext:realElement {
    [realElement _popFromContextWithAssociations:associations keys:keys];
}

+ (id)getComponentNamed:(NSString *)aName;
{
    return [WOApp pageWithName:aName];
}

- (id)initWithName:(NSString *)aName associations:(NSDictionary *)someAssociations template:(WOElement *)aTemplate;
{
    NSArray *keyArray;
    NSString *key;
    unsigned int index, count;

    [super initWithName:aName associations:someAssociations template:aTemplate];

    if (!(realComponentAssociation = [someAssociations objectForKey:@"realComponent"])) {
        [self release];
	[NSException raise:NSInvalidArgumentException 
		    format:@"'realComponent' attribute missing"];
    }

    [realComponentAssociation retain];

    if ((dynamicAssociationsAssociation = [someAssociations objectForKey:@"dynamicAssociations"]))
        [dynamicAssociationsAssociation retain];
    
    keyArray = [someAssociations allKeys];
    count = [keyArray count];

    if (((count > 1) && (!dynamicAssociationsAssociation)) || ((count > 2) && (dynamicAssociationsAssociation))) {
        NSMutableArray *tempKeys;
        NSMutableArray *tempAssociations;

        tempKeys = [[NSMutableArray alloc] init];
        tempAssociations = [[NSMutableArray alloc] init];

        for (index = 0; index < count; index++) {
            key = [keyArray objectAtIndex:index];

            if ([key isEqualToString:@"dynamicAssociations"]) {
                NSDictionary *dynamicAssociations;
                NSArray *dynamicAssociationKeys;
                unsigned int dynamicAssociationIndex, dynamicAssociationCount;

                dynamicAssociations = [dynamicAssociationsAssociation value];
                dynamicAssociationKeys = [dynamicAssociations allKeys];
                dynamicAssociationCount = [dynamicAssociationKeys count];

                for (dynamicAssociationIndex = 0; dynamicAssociationIndex < dynamicAssociationCount; dynamicAssociationIndex++) {
                    NSString *dynamicAssociationKey;

                    dynamicAssociationKey = [dynamicAssociationKeys objectAtIndex:dynamicAssociationIndex];

                    [tempKeys addObject:dynamicAssociationKey];
                    [tempAssociations addObject:[dynamicAssociations objectForKey:dynamicAssociationKey]];
                }
            } else if (![key isEqualToString:@"realComponent"]) {
                [tempKeys addObject:key];
                [tempAssociations addObject:[someAssociations objectForKey:key]];
            }
        }

        keys = [[NSArray alloc] initWithArray:tempKeys];
        associations = [[NSArray alloc] initWithArray:tempAssociations];

        [tempKeys release];
        [tempAssociations release];
    } else {
        keys = [[NSArray alloc] init];
        associations = [[NSArray alloc] init];
    }          
    return self;
}

- (void)dealloc;
{
    [realComponentAssociation release];
    [dynamicAssociationsAssociation release];
    [keys release];
    [associations release];

    [super dealloc];
}

- (void)takeValuesFromRequest:(WORequest *)aRequest inContext:(WOContext *)aContext;
{
    id realElement;

    realElement = [realComponentAssociation value];

    [self _pushComponentInContext:realElement];
    [realElement takeValuesFromRequest:aRequest inContext:aContext];
    [self _popComponentFromContext:realElement];
}

- (WOElement *)invokeActionForRequest:(WORequest *)aRequest inContext:(WOContext *)aContext;
{
    id         realElement;
    WOElement *returnedElement;

    realElement = [realComponentAssociation value];
    
    [self _pushComponentInContext:realElement];
    returnedElement = [realElement invokeActionForRequest:aRequest inContext:aContext];
    [self _popComponentFromContext:realElement];

    return returnedElement;
}

- (void)appendToResponse:(WOResponse *)aResponse inContext:(WOContext *)aContext;
{
    id realElement;

    if ([[aContext request] isFromClientComponent])
        return;

    realElement = [realComponentAssociation value];

    [self _pushComponentInContext:realElement];
    [realElement appendToResponse:aResponse inContext:aContext];
    [self _popComponentFromContext:realElement];
}


@end
